@@ -41,7 +41,7 @@ namespace serialization {
 
 /// Boost Serialization library support for units.
 template<class Archive,class System,class Dim>
-inline void serialize(Archive& ar,boost::units::unit<Dim,System>&,const unsigned int /*version*/)
+inline void serialize(Archive& /*ar*/,boost::units::unit<Dim,System>&,const unsigned int /*version*/)
 { }
 
 /// Boost Serialization library support for quantities.
@@ -649,17 +649,55 @@ inline void do_print(std::basic_ostream<Char, Traits>& os, const char* s)
 
 // For automatically applying the appropriate prefixes.
 
+}
+
+#ifdef BOOST_UNITS_DOXYGEN
+
+/// ADL customization point for automatic prefixing.
+/// Returns a non-negative value.  Implemented as std::abs
+/// for built-in types.
+template<class T>
+double autoprefix_norm(const T& arg);
+
+#else
+
+template<class T, bool C = boost::is_arithmetic<T>::value>
+struct autoprefix_norm_impl;
+
+template<class T>
+struct autoprefix_norm_impl<T, true>
+{
+    typedef double type;
+    static double call(const T& arg) { return std::abs(arg); }
+};
+
+template<class T>
+struct autoprefix_norm_impl<T, false>
+{
+    typedef one type;
+    static one call(const T&) { return one(); }
+};
+
+template<class T>
+typename autoprefix_norm_impl<T>::type autoprefix_norm(const T& arg)
+{
+    return autoprefix_norm_impl<T>::call(arg);
+}
+
+#endif
+
+namespace detail {
+
 template<class End, class Prev, class T, class F>
-bool find_matching_scale_impl(End, End, Prev, T, F)
+bool find_matching_scale_impl(End, End, Prev, T, double, F)
 {
     return false;
 }
 
 template<class Begin, class End, class Prev, class T, class F>
-bool find_matching_scale_impl(Begin, End end, Prev prev, T t, F f)
+bool find_matching_scale_impl(Begin, End end, Prev prev, T t, double x, F f)
 {
-    using std::abs;
-    if(Begin::item::value() > abs(t)) {
+    if(Begin::item::value() > x) {
         f(prev, t);
         return true;
     } else {
@@ -668,32 +706,32 @@ bool find_matching_scale_impl(Begin, End end, Prev prev, T t, F f)
             end,
             typename Begin::item(),
             t,
+            x,
             f
         );
     }
 }
 
 template<class End, class T, class F>
-bool find_matching_scale_i(End, End, T, F)
+bool find_matching_scale_i(End, End, T, double, F)
 {
     return false;
 }
 
 template<class Begin, class End, class T, class F>
-bool find_matching_scale_i(Begin, End end, T t, F f)
+bool find_matching_scale_i(Begin, End end, T t, double x, F f)
 {
-    using std::abs;
-    if(Begin::item::value() > abs(t)) {
+    if(Begin::item::value() > x) {
         return false;
     } else {
-        return detail::find_matching_scale_impl(typename Begin::next(), end, typename Begin::item(), t, f);
+        return detail::find_matching_scale_impl(typename Begin::next(), end, typename Begin::item(), t, x, f);
     }
 }
 
 template<class Scales, class T, class F>
-bool find_matching_scale(T t, F f)
+bool find_matching_scale(T t, double x, F f)
 {
-    return detail::find_matching_scale_i(Scales(), dimensionless_type(), t, f);
+    return detail::find_matching_scale_i(Scales(), dimensionless_type(), t, x, f);
 }
 
 typedef list<scale<10, static_rational<-24> >,
@@ -723,7 +761,9 @@ typedef list<scale<2, static_rational<10> >,
         list<scale<2, static_rational<50> >,
         list<scale<2, static_rational<60> >,
         list<scale<2, static_rational<70> >,
-        dimensionless_type> > > > > > > binary_prefixes;
+        list<scale<2, static_rational<80> >,
+        list<scale<2, static_rational<90> >,
+        dimensionless_type> > > > > > > > > binary_prefixes;
 
 template<class Os, class Quantity>
 struct print_default_t {
@@ -814,7 +854,7 @@ template<class Prefixes, class CharT, class Traits, class Unit, class T, class F
 void do_print_prefixed_impl(std::basic_ostream<CharT, Traits>& os, const quantity<Unit, T>& q, F default_)
 {
     bool prefixed;
-    if(detail::find_matching_scale<Prefixes>(q.value(), detail::print_scale(os, prefixed))) {
+    if(detail::find_matching_scale<Prefixes>(q.value(), autoprefix_norm(q.value()), detail::print_scale(os, prefixed))) {
         if(prefixed) {
             switch(units::get_format(os)) {
                 case symbol_fmt: do_print(os, maybe_parenthesize(Unit(), format_symbol_impl())); break;
@@ -923,6 +963,21 @@ void do_print_prefixed(std::basic_ostream<CharT, Traits>& os, const quantity<Uni
     detail::print_default(os, q)();
 }
 
+template<class Prefixes, class CharT, class Traits, class Unit, class T>
+void maybe_print_prefixed(std::basic_ostream<CharT, Traits>& os, const quantity<Unit, T>& q, mpl::true_)
+{
+    detail::do_print_prefixed<Prefixes>(os, q);
+}
+
+template<class Prefixes, class CharT, class Traits, class Unit, class T>
+void maybe_print_prefixed(std::basic_ostream<CharT, Traits>& os, const quantity<Unit, T>& q, mpl::false_)
+{
+    detail::print_default(os, q)();
+}
+
+inline mpl::true_ test_norm(double) { return mpl::true_(); }
+inline mpl::false_ test_norm(one) { return mpl::false_(); }
+
 } // namespace detail
 
 template<class Dimension,class System>
@@ -994,11 +1049,11 @@ inline std::basic_ostream<Char, Traits>& operator<<(std::basic_ostream<Char, Tra
     }
     else if (units::get_autoprefix(os) == autoprefix_engineering)
     {
-        detail::do_print_prefixed<detail::engineering_prefixes>(os, q);
+        detail::maybe_print_prefixed<detail::engineering_prefixes>(os, q, detail::test_norm(autoprefix_norm(q.value())));
     }
     else if (units::get_autoprefix(os) == autoprefix_binary)
     {
-        detail::do_print_prefixed<detail::binary_prefixes>(os, q);
+        detail::maybe_print_prefixed<detail::binary_prefixes>(os, q, detail::test_norm(autoprefix_norm(q.value())));
     }
     else
     {
