@@ -297,7 +297,7 @@ void Parser::Terminate_Tokenizer()
 
 void Parser::Get_Token ()
 {
-	register int c,c2;
+	int c,c2;
 	int col;
 
 	if (Token.Unget_Token)
@@ -702,7 +702,7 @@ void Parser::Unget_Token ()
 
 bool Parser::Skip_Spaces()
 {
-	register int c;
+	int c;
 
 	while(true)
 	{
@@ -744,7 +744,7 @@ bool Parser::Skip_Spaces()
 
 int Parser::Parse_C_Comments()
 {
-	register int c, c2;
+	int c, c2;
 	bool End_Of_Comment = false;
 
 	while(!End_Of_Comment)
@@ -990,7 +990,7 @@ inline void Parser::End_String_Fast()
 
 void Parser::Parse_String_Literal()
 {
-	register int c;
+	int c;
 	int col = Echo_Indx;
 
 	Begin_String();
@@ -1066,8 +1066,8 @@ void Parser::Parse_String_Literal()
 
 bool Parser::Read_Float()
 {
-	register int c, Phase;
-	register bool Finished;
+	int c, Phase;
+	bool Finished;
 	int col = Echo_Indx;
 
 	Finished = false;
@@ -1252,7 +1252,7 @@ bool Parser::Read_Float()
 
 void Parser::Read_Symbol()
 {
-	register int c;
+	int c;
 	int Local_Index,i,j,k;
 	POV_ARRAY *a;
 	SYM_ENTRY *Temp_Entry;
@@ -1439,7 +1439,7 @@ inline void Parser::Write_Token (TOKEN Token_Id, int col)
 
 const char *Parser::Get_Token_String (TOKEN Token_Id)
 {
-	register int i;
+	int i;
 
 	for (i = 0 ; i < LAST_TOKEN ; i++)
 		if (Reserved_Words[i].Token_Number == Token_Id)
@@ -1527,7 +1527,7 @@ char *Parser::Get_Reserved_Words (const char *additional_words)
 
 int Parser::Echo_getc()
 {
-	register int c;
+	int c;
 
 	if((Input_File == NULL) || (Input_File->In_File == NULL) || (c = Input_File->In_File->getchar()) == EOF)
 	{
@@ -3762,7 +3762,7 @@ int Parser::Parse_Ifdef_Param (void)
 	SYM_ENTRY *Entry;
 	int retval = false;
 	int i,j,k;
-	register int c;
+	int c;
 	DBL val;
 	POV_ARRAY *a;
 
