@@ -1,46 +1,52 @@
-/****************************************************************************
- *                  LIGHTGRP.CPP
+/*******************************************************************************
+ * lightgrp.cpp
  *
  * Implements light group utility functions.
  *
- * from Persistence of Vision(tm) Ray Tracer version 3.6.
- * Copyright 1991-2003 Persistence of Vision Team
- * Copyright 2003-2004 Persistence of Vision Raytracer Pty. Ltd.
- *---------------------------------------------------------------------------
- * NOTICE: This source code file is provided so that users may experiment
- * with enhancements to POV-Ray and to port the software to platforms other
- * than those supported by the POV-Ray developers. There are strict rules
- * regarding how you are permitted to use this file. These rules are contained
- * in the distribution and derivative versions licenses which should have been
- * provided with this file.
+ * ---------------------------------------------------------------------------
+ * Persistence of Vision Ray Tracer ('POV-Ray') version 3.7.
+ * Copyright 1991-2013 Persistence of Vision Raytracer Pty. Ltd.
  *
- * These licences may be found online, linked from the end-user license
- * agreement that is located at http://www.povray.org/povlegal.html
- *---------------------------------------------------------------------------
- * This program is based on the popular DKB raytracer version 2.12.
+ * POV-Ray is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * POV-Ray is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * ---------------------------------------------------------------------------
+ * POV-Ray is based on the popular DKB raytracer version 2.12.
  * DKBTrace was originally written by David K. Buck.
  * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
- *---------------------------------------------------------------------------
- * $File: //depot/povray/3.6-release/source/lightgrp.cpp $
- * $Revision: #3 $
- * $Change: 3032 $
- * $DateTime: 2004/08/02 18:43:41 $
+ * ---------------------------------------------------------------------------
+ * $File: //depot/public/povray/3.x/source/lightgrp.cpp $
+ * $Revision: #1 $
+ * $Change: 6069 $
+ * $DateTime: 2013/11/06 11:59:40 $
  * $Author: chrisc $
- * $Log$
- *****************************************************************************/
+ *******************************************************************************/
 
-#include "frame.h"
-#include "lightgrp.h"
-#include "povray.h"
-#include "point.h"
-#include "objects.h"
-#include "pov_util.h"
-#include "csg.h"
-#include "photons.h"
+// frame.h must always be the first POV file included (pulls in platform config)
+#include "backend/frame.h"
 
-BEGIN_POV_NAMESPACE
+#include "backend/scene/objects.h"
+#include "backend/shape/csg.h"
+#include "backend/lighting/point.h"
 
-void Promote_Local_Lights_Recursive(COMPOUND_OBJECT *Object, LIGHT_SOURCE *Lights);
+#include "lightgrp.h" // TODO
+
+// this must be the last file included
+#include "base/povdebug.h"
+
+namespace pov
+{
+
+void Promote_Local_Lights_Recursive(CompoundObject *Object, vector<LightSource *>& Lights);
 
 
 /*****************************************************************************
@@ -82,35 +88,21 @@ void Promote_Local_Lights_Recursive(COMPOUND_OBJECT *Object, LIGHT_SOURCE *Light
 
 void Promote_Local_Lights(CSG *Object)
 {
-	LIGHT_SOURCE *lights = NULL;
-	OBJECT *curObject = NULL;
-	int light_counter = 0;
-	int object_counter = 0;
+	vector<LightSource *> lights;
 
 	if(Object == NULL)
 		return;
 
 	// find all light sources in the light group and connect them to form a list
-	for(curObject = Object->Children, light_counter = 0, object_counter = 0;
-	    curObject != NULL;
-	    curObject = curObject->Sibling)
+	int light_counter = 0;
+	int object_counter = 0;
+	for(vector<ObjectPtr>::iterator curObject = Object->children.begin();
+	    curObject != Object->children.end();
+	    curObject++)
 	{
-		if((curObject->Type & LIGHT_GROUP_LIGHT_OBJECT) == LIGHT_GROUP_LIGHT_OBJECT)
+		if(((*curObject)->Type & LIGHT_GROUP_LIGHT_OBJECT) == LIGHT_GROUP_LIGHT_OBJECT)
 		{
-			// linked list of light sources
-			if(lights == NULL)
-			{
-				lights = (LIGHT_SOURCE *)curObject;
-				lights->Next_Light_Source = NULL;
-			}
-			else
-			{
-				LIGHT_SOURCE *l = (LIGHT_SOURCE *)curObject;
-
-				l->Next_Light_Source = lights;
-				lights = l;
-			}
-
+			lights.push_back((LightSource *)(*curObject));
 			light_counter++;
 		}
 		else
@@ -121,13 +113,13 @@ void Promote_Local_Lights(CSG *Object)
 	// user should know there are no lights (also it will continue to work as a union)
 	if(light_counter <= 0)
 	{
-		Warning(0, "No light source(s) found in light group.");
+;// TODO MESSAGE		Warning(0, "No light source(s) found in light group.");
 		return;
 	}
 	// if no objects have been found nothing will happen at all (the light group is only wasting memory)
 	if(object_counter <= 0)
 	{
-		Warning(0, "No object(s) found in light group.");
+;// TODO MESSAGE		Warning(0, "No object(s) found in light group.");
 		return;
 	}
 
@@ -135,7 +127,7 @@ void Promote_Local_Lights(CSG *Object)
 	Object->LLights = lights;
 
 	// promote light recursively to all other objects in the CSG union
-	Promote_Local_Lights_Recursive((COMPOUND_OBJECT *)Object, lights);
+	Promote_Local_Lights_Recursive((CompoundObject *)Object, lights);
 }
 
 /*****************************************************************************
@@ -172,39 +164,29 @@ void Promote_Local_Lights(CSG *Object)
 *
 ******************************************************************************/
 
-void Promote_Local_Lights_Recursive(COMPOUND_OBJECT *Object, LIGHT_SOURCE *Lights)
+void Promote_Local_Lights_Recursive(CompoundObject *Object, vector<LightSource *>& Lights)
 {
-	OBJECT *curObject = NULL;
+	ObjectPtr curObject = NULL;
 
-	for(curObject = Object->Children;
-	    curObject != NULL;
-	    curObject = curObject->Sibling)
+	for(vector<ObjectPtr>::iterator curObject = Object->children.begin();
+	    curObject != Object->children.end();
+	    curObject++)
 	{
-		if(curObject->LLights != NULL)
+		if(!(*curObject)->LLights.empty())
 		{
-			LIGHT_SOURCE *l = curObject->LLights;
-
-			// the code below is using (l != Lights) as safety
-			// check - it should never be true if my logic works [trf]
-
-			// find end of current object's light list
-			while((l->Next_Light_Source != NULL) && (l != Lights))
-				l = l->Next_Light_Source;
-
-			// add inherited light list
-			if(l != Lights)
-				l->Next_Light_Source = Lights;
+			for(vector<LightSource *>::iterator i = Lights.begin(); i != Lights.end(); i++)
+				(*curObject)->LLights.push_back(*i);
 		}
-		else if((curObject->Type & IS_COMPOUND_OBJECT) == IS_COMPOUND_OBJECT)
+		else if(((*curObject)->Type & IS_COMPOUND_OBJECT) == IS_COMPOUND_OBJECT)
 		{
 			// allow easy promotion of lights (if this is part of another light group)
-			curObject->LLights = Lights;
+			(*curObject)->LLights = Lights;
 
-			Promote_Local_Lights_Recursive((COMPOUND_OBJECT *)curObject, Lights);
+			Promote_Local_Lights_Recursive((CompoundObject *)(*curObject), Lights);
 		}
 		else
 		{
-			curObject->LLights = Lights;
+			(*curObject)->LLights = Lights;
 		}
 	}
 }
@@ -249,26 +231,24 @@ void Promote_Local_Lights_Recursive(COMPOUND_OBJECT *Object, LIGHT_SOURCE *Light
 *
 ******************************************************************************/
 
-bool Check_Photon_Light_Group(OBJECT* Object)
+bool Check_Photon_Light_Group(ConstObjectPtr Object)
 {
-  if(photonOptions.Light_Is_Global)
-  {
-    if ((Object->Flags & NO_GLOBAL_LIGHTS_FLAG) == NO_GLOBAL_LIGHTS_FLAG)
-      return false;
-    else
-      return true;
-  }
-  else
-  {
-    LIGHT_SOURCE *Test_Light;
-    for(Test_Light = Object->LLights;
-        Test_Light != NULL;
-        Test_Light = Test_Light->Next_Light_Source)
-    {
-      if (Test_Light == photonOptions.Light) return true;
-    }
-    return false;
-  }
+/* TODO FIXME	if(photonOptions.Light_Is_Global)
+	{
+		if((Object->Flags & NO_GLOBAL_LIGHTS_FLAG) == NO_GLOBAL_LIGHTS_FLAG)
+			return false;
+		else
+			return true;
+	}
+	else
+	{
+		for(vector<LightSource *>::iterator Test_Light = Object->LLights.begin(); Test_Light != Object->LLights.end(); Test_Light++)
+		{
+			if(*Test_Light == photonOptions.Light)
+				return true;
+		} */
+		return true;
+// 	}
 }
 
-END_POV_NAMESPACE
+}
