@@ -1,46 +1,49 @@
-/****************************************************************************
- *               pov_mem.cpp
+/*******************************************************************************
+ * pov_mem.cpp
  *
  * This module contains the code for our own memory allocation/deallocation,
  * providing memory tracing, statistics, and garbage collection options.
  *
- * from Persistence of Vision(tm) Ray Tracer version 3.6.
- * Copyright 1991-2003 Persistence of Vision Team
- * Copyright 2003-2004 Persistence of Vision Raytracer Pty. Ltd.
- *---------------------------------------------------------------------------
- * NOTICE: This source code file is provided so that users may experiment
- * with enhancements to POV-Ray and to port the software to platforms other
- * than those supported by the POV-Ray developers. There are strict rules
- * regarding how you are permitted to use this file. These rules are contained
- * in the distribution and derivative versions licenses which should have been
- * provided with this file.
+ * ---------------------------------------------------------------------------
+ * Persistence of Vision Ray Tracer ('POV-Ray') version 3.7.
+ * Copyright 1991-2013 Persistence of Vision Raytracer Pty. Ltd.
  *
- * These licences may be found online, linked from the end-user license
- * agreement that is located at http://www.povray.org/povlegal.html
- *---------------------------------------------------------------------------
- * This program is based on the popular DKB raytracer version 2.12.
+ * POV-Ray is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * POV-Ray is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * ---------------------------------------------------------------------------
+ * POV-Ray is based on the popular DKB raytracer version 2.12.
  * DKBTrace was originally written by David K. Buck.
  * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
- *---------------------------------------------------------------------------
- * $File: //depot/povray/3.6-release/source/pov_mem.cpp $
- * $Revision: #2 $
- * $Change: 2939 $
- * $DateTime: 2004/07/04 13:43:26 $
- * $Author: root $
- * $Log$
- *****************************************************************************/
+ * ---------------------------------------------------------------------------
+ * $File: //depot/public/povray/3.x/source/pov_mem.cpp $
+ * $Revision: #1 $
+ * $Change: 6069 $
+ * $DateTime: 2013/11/06 11:59:40 $
+ * $Author: chrisc $
+ *******************************************************************************/
 
-#include "frame.h"
-#include "povproto.h"           /* Error() */
+// frame.h must always be the first POV file included (pulls in platform config)
+#include "backend/frame.h"
 
 #include "pov_mem.h"
-#include "parse.h"              /* MAError() */
+#include "backend/parser/parse.h"              /* MAError() */
 #include "povray.h"             /* stats[] global var */
-#include "optout.h"
-#include "pov_util.h"
-#include "povmsend.h"
 
-BEGIN_POV_NAMESPACE
+// this must be the last file included
+#include "base/povdebug.h"
+
+namespace pov
+{
 
 /************************************************************************
 * AUTHOR
@@ -277,18 +280,18 @@ typedef struct MemStats_Struct MEMSTATS;
 
 struct MemStats_Struct
 {
-  size_t     smallest_alloc;    /* smallest # of bytes in one malloc() */
-  size_t     largest_alloc;     /* largest # of bytes in one malloc() */
-  size_t     current_mem_usage; /* current total # of bytes allocated */
-  size_t     largest_mem_usage; /* peak total # of bytes allocated */
+	size_t     smallest_alloc;    /* smallest # of bytes in one malloc() */
+	size_t     largest_alloc;     /* largest # of bytes in one malloc() */
+	size_t     current_mem_usage; /* current total # of bytes allocated */
+	size_t     largest_mem_usage; /* peak total # of bytes allocated */
 #if (MEM_STATS>=2)
-  /* could add a running average size too, someday */
-  long int   total_allocs;      /* total # of alloc calls */
-  long int   total_frees;       /* total # of free calls */
-  const char *smallest_file;     /* file name of largest alloc */
-  int        smallest_line;     /* file line of largest alloc */
-  const char *largest_file;      /* file name of largest alloc */
-  int        largest_line;      /* file line of largest alloc */
+	/* could add a running average size too, someday */
+	long int   total_allocs;      /* total # of alloc calls */
+	long int   total_frees;       /* total # of free calls */
+	const char *smallest_file;    /* file name of largest alloc */
+	int        smallest_line;     /* file line of largest alloc */
+	const char *largest_file;     /* file name of largest alloc */
+	int        largest_line;      /* file line of largest alloc */
 #endif
 };
 
@@ -307,35 +310,34 @@ static void mem_stats_free (size_t nbytes);
 void mem_init()
 {
 #if defined(MEM_RECLAIM)
-  num_nodes = 0;
-  poolno = 0;
-  memlist = NULL;
+	num_nodes = 0;
+	poolno = 0;
+	memlist = NULL;
 #endif
 #if defined(MEM_GUARD)
-  mem_guard_string_len = strlen(mem_guard_string);
+	mem_guard_string_len = strlen(mem_guard_string);
 #endif
 #if defined(MEM_PREFILL)
-  mem_prefill_string_len = strlen(mem_prefill_string);
-  mem_clear_string_len = strlen(mem_clear_string);
+	mem_prefill_string_len = strlen(mem_prefill_string);
+	mem_clear_string_len = strlen(mem_clear_string);
 #endif
 #if defined(MEM_STATS)
-  mem_stats_init();
+	mem_stats_init();
 #endif
-  leak_msg = false;
+	leak_msg = false;
 }
 
-
 #if defined(MEM_TAG)
 /****************************************************************************/
 /* return true if pointer is non-null and has a valid tag */
 static int mem_check_tag(MEMNODE *node)
 {
-  int isOK = false;
+	int isOK = false;
 
-  if (node != NULL)
-    if (node->tag == MEMTAG_VALUE)
-      isOK = true;
-  return isOK;
+	if (node != NULL)
+		if (node->tag == MEMTAG_VALUE)
+			isOK = true;
+	return isOK;
 }
 
 #endif /* MEM_TAG */
@@ -344,317 +346,317 @@ static int mem_check_tag(MEMNODE *node)
 /****************************************************************************/
 void *pov_malloc(size_t size, const char *file, int line, const char *msg)
 {
-  void *block;
-  size_t totalsize;
+	void *block;
+	size_t totalsize;
 #if defined(MEM_HEADER)
-  MEMNODE *node;
+	MEMNODE *node;
 #endif
 
 #if defined(MEM_PREFILL) || defined(MEM_GUARD)
-  char *memptr;
-  size_t i;
+	char *memptr;
+	size_t i;
 #endif
 
 #if defined(MEM_HEADER)
-  if (size == 0)
-  {
-    Error("Attempt to malloc zero size block (File: %s Line: %d).", file, line);
-  }
+	if (size == 0)
+	{
+// TODO MESSAGE		Error("Attempt to malloc zero size block (File: %s Line: %d).", file, line);
+	}
 #endif
 
-  totalsize = size + NODESIZE + (MEM_GUARD_SIZE * 2); /* number of bytes allocated in OS */
+	totalsize = size + NODESIZE + (MEM_GUARD_SIZE * 2); /* number of bytes allocated in OS */
 
-  block = (void *)MALLOC(totalsize);
+	block = (void *)MALLOC(totalsize);
 
-  if (block == NULL)
-    MAError(msg, size);
+	if (block == NULL)
+		throw std::bad_alloc();; // TODO FIXME !!! // Parser::MAError(msg, (int)size);
 
 #if defined(MEM_HEADER)
-  node = (MEMNODE *) block;
+	node = (MEMNODE *) block;
 #endif
 
 #if defined(MEM_TAG)
-  node->tag = MEMTAG_VALUE;
+	node->tag = MEMTAG_VALUE;
 #endif
 
 #if defined(MEM_TRACE) || defined(MEM_STATS)
-  node->size = totalsize;
+	node->size = totalsize;
 #endif
 #if defined(MEM_TRACE)
-  node->file = file;
-  node->line = line;
+	node->file = file;
+	node->line = line;
 #endif
 
 #if defined(MEM_PREFILL)
-  memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
-  for(i = 0; i < size; i++)
-     memptr[i] = mem_prefill_string[i % mem_prefill_string_len];
+	memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
+	for(i = 0; i < size; i++)
+		memptr[i] = mem_prefill_string[i % mem_prefill_string_len];
 #endif
 
 #if defined(MEM_GUARD)
-  memptr = (char *)block + NODESIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-     memptr[i] = mem_guard_string[i % mem_guard_string_len];
-  memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-     memptr[i] = mem_guard_string[i % mem_guard_string_len];
+	memptr = (char *)block + NODESIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+		memptr[i] = mem_guard_string[i % mem_guard_string_len];
+	memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+		memptr[i] = mem_guard_string[i % mem_guard_string_len];
 #endif
 
 #if defined(MEM_RECLAIM)
-  add_node(node);
+	add_node(node);
 #endif
 
 #if defined(MEM_STATS)
-  mem_stats_alloc(totalsize, file, line);
+	mem_stats_alloc(totalsize, file, line);
 #endif
 
-  return (void *)((char *)block + NODESIZE + MEM_GUARD_SIZE);
+	return (void *)((char *)block + NODESIZE + MEM_GUARD_SIZE);
 }
 
 
 /****************************************************************************/
 void *pov_calloc(size_t nitems, size_t size, const char *file, int line, const char *msg)
 {
-  void *block;
-  size_t actsize;
+	void *block;
+	size_t actsize;
 
-  actsize = nitems * size;
+	actsize = nitems * size;
 
 #if defined(MEM_HEADER)
-  if (actsize == 0)
-  {
-    Error("Attempt to calloc zero size block (File: %s Line: %d).", file, line);
-  }
+	if (actsize == 0)
+	{
+// TODO MESSAGE		Error("Attempt to calloc zero size block (File: %s Line: %d).", file, line);
+	}
 #endif
 
-  block = (void *)pov_malloc(actsize, file, line, msg);
+	block = (void *)pov_malloc(actsize, file, line, msg);
 
-  if (block != NULL)
-    memset(block, 0, actsize);
+	if (block != NULL)
+		memset(block, 0, actsize);
 
-  return block;
+	return block;
 }
 
 
 /****************************************************************************/
 void *pov_realloc(void *ptr, size_t size, const char *file, int line, const char *msg)
 {
-  void *block;
+	void *block;
 #if defined(MEM_STATS)
-  size_t oldsize;
+	size_t oldsize;
 #endif
 
 #if defined(MEM_HEADER)
-  MEMNODE *node;
+	MEMNODE *node;
 #endif
 
 #if defined(MEM_RECLAIM)
-  MEMNODE *prev;
-  MEMNODE *next;
+	MEMNODE *prev;
+	MEMNODE *next;
 #endif
 
 #if defined(MEM_PREFILL) || defined(MEM_GUARD)
-  char *memptr;
-  size_t i;
+	char *memptr;
+	size_t i;
 #endif
 
-  if (size == 0)
-  {
-    if (ptr)
-      pov_free(ptr, file, line);
-    return NULL;
-  }
-  else if (ptr == NULL)
-    return pov_malloc(size, file, line, msg);
+	if (size == 0)
+	{
+		if (ptr)
+			pov_free(ptr, file, line);
+		return NULL;
+	}
+	else if (ptr == NULL)
+		return pov_malloc(size, file, line, msg);
 
-  block = (void *)((char *)ptr - NODESIZE - MEM_GUARD_SIZE);
+	block = (void *)((char *)ptr - NODESIZE - MEM_GUARD_SIZE);
 
 #if defined(MEM_GUARD)
-  memptr = (char *)block + NODESIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-  {
-     if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
-     {
-        Warning(0, "Damaged start guard detected in resized block (File: %s Line: %d).", file, line);
-        break;
-     }
-  }
-  memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-  {
-     if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
-     {
-        Warning(0, "Damaged end guard detected in resized block (File: %s Line: %d).", file, line);
-        break;
-     }
-  }
+	memptr = (char *)block + NODESIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+	{
+		if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
+		{
+			Warning(0, "Damaged start guard detected in resized block (File: %s Line: %d).", file, line);
+			break;
+		}
+	}
+	memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+	{
+		if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
+		{
+			Warning(0, "Damaged end guard detected in resized block (File: %s Line: %d).", file, line);
+			break;
+		}
+	}
 #endif
 
 #if defined(MEM_HEADER)
-  node = (MEMNODE *) block;
+	node = (MEMNODE *) block;
 #endif
 
 #if defined(MEM_TAG)
-  if (node->tag != MEMTAG_VALUE)
-    Error("Attempt to realloc invalid block (File: %s Line: %d).", file, line);
+	if (node->tag != MEMTAG_VALUE)
+		Error("Attempt to realloc invalid block (File: %s Line: %d).", file, line);
 
-  node->tag = ~node->tag;
+	node->tag = ~node->tag;
 #endif
 
 #if defined(MEM_RECLAIM)
-  prev = node->prev;
-  next = node->next;
+	prev = node->prev;
+	next = node->next;
 #endif
 
 #if defined(MEM_STATS)
-  oldsize = ((MEMNODE *)block)->size;
-
-  #if defined(MEM_PREFILL)
-    memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
-    for(i = size; i < oldsize - NODESIZE - (MEM_GUARD_SIZE * 2); i++)
-       memptr[i] = mem_clear_string[i % mem_clear_string_len];
-  #endif
+	oldsize = ((MEMNODE *)block)->size;
 #endif
 
-  block = (void *)REALLOC(block, NODESIZE + (MEM_GUARD_SIZE * 2) + size);
+#if defined(MEM_PREFILL)
+	memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
+	for(i = size; i < oldsize - NODESIZE - (MEM_GUARD_SIZE * 2); i++)
+		memptr[i] = mem_clear_string[i % mem_clear_string_len];
+#endif
 
-  if (block == NULL)
-    MAError(msg, size);
+	block = (void *)REALLOC(block, NODESIZE + (MEM_GUARD_SIZE * 2) + size);
+
+	if (block == NULL)
+		throw std::bad_alloc(); // TODO FIXME !!! // Parser::MAError(msg, (int)size);
 
 #if defined(MEM_STATS)
-  /* REALLOC does an implied FREE... */
-  mem_stats_free(oldsize);
-  /* ...and an implied MALLOC... */
-  mem_stats_alloc(NODESIZE + (MEM_GUARD_SIZE * 2) + size, file, line);
-
-  #if defined(MEM_PREFILL)
-    memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
-    for(i = oldsize - NODESIZE - (MEM_GUARD_SIZE * 2); i < size; i++)
-       memptr[i] = mem_prefill_string[i % mem_prefill_string_len];
-  #endif
+	/* REALLOC does an implied FREE... */
+	mem_stats_free(oldsize);
+	/* ...and an implied MALLOC... */
+	mem_stats_alloc(NODESIZE + (MEM_GUARD_SIZE * 2) + size, file, line);
+#endif
 
+#if defined(MEM_PREFILL)
+	memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
+	for(i = oldsize - NODESIZE - (MEM_GUARD_SIZE * 2); i < size; i++)
+		memptr[i] = mem_prefill_string[i % mem_prefill_string_len];
 #endif
 
 #if defined(MEM_HEADER)
-  node = (MEMNODE *) block;
+	node = (MEMNODE *) block;
 #endif
 
 #if defined(MEM_TAG)
-  node->tag = MEMTAG_VALUE;
+	node->tag = MEMTAG_VALUE;
 #endif
 
 #if defined(MEM_TRACE) || defined(MEM_STATS)
-  node->size = size + NODESIZE + (MEM_GUARD_SIZE * 2);
+	node->size = size + NODESIZE + (MEM_GUARD_SIZE * 2);
 #endif
 #if defined(MEM_TRACE)
-  node->file = file;
-  node->line = line;
+	node->file = file;
+	node->line = line;
 #endif
 
 #if defined(MEM_GUARD)
-  memptr = (char *)block + NODESIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-     memptr[i] = mem_guard_string[i % mem_guard_string_len];
-  memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-     memptr[i] = mem_guard_string[i % mem_guard_string_len];
+	memptr = (char *)block + NODESIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+		memptr[i] = mem_guard_string[i % mem_guard_string_len];
+	memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+		memptr[i] = mem_guard_string[i % mem_guard_string_len];
 #endif
 
 #if defined(MEM_RECLAIM)
-  if (prev == NULL)
-    memlist = node;
-  else
-    prev->next = node;
-  if (node->next != NULL)
-    node->next->prev = node;
-  if (next != NULL)
-    next->prev = node;
+	if (prev == NULL)
+		memlist = node;
+	else
+		prev->next = node;
+	if (node->next != NULL)
+		node->next->prev = node;
+	if (next != NULL)
+		next->prev = node;
 #endif
 
-  return (void *)((char *)block + NODESIZE + MEM_GUARD_SIZE);
+	return (void *)((char *)block + NODESIZE + MEM_GUARD_SIZE);
 }
 
 
 /****************************************************************************/
 void pov_free(void *ptr, const char *file, int line)
 {
-  void *block;
+	void *block;
 
 #if defined(MEM_HEADER)
-  MEMNODE *node;
+	MEMNODE *node;
 #endif
 
 #if defined(MEM_PREFILL) || defined(MEM_GUARD)
-  char *memptr;
-  size_t size;
-  size_t i;
+	char *memptr;
+	size_t size;
+	size_t i;
 #endif
 
-  if (ptr == NULL)
-    Error("Attempt to free NULL pointer (File: %s Line: %d).", file, line);
+	if (ptr == NULL)
+		throw pov_base::Exception(NULL, file, (unsigned int)line, "Attempt to free NULL pointer.");
 
-  block = (void *)((char *)ptr - NODESIZE - MEM_GUARD_SIZE);
+	block = (void *)((char *)ptr - NODESIZE - MEM_GUARD_SIZE);
 
 #if defined(MEM_HEADER)
-  node = (MEMNODE *) block;
+	node = (MEMNODE *) block;
 #endif
 
 #if defined(MEM_TAG)
-  #if defined(MEM_PREFILL)
-    size = ((MEMNODE *)block)->size;
-    memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
-    for(i = 0; i < size - NODESIZE - (MEM_GUARD_SIZE * 2); i++)
-       memptr[i] = mem_clear_string[i % mem_clear_string_len];
-  #endif
-
-  if (node->tag == ~MEMTAG_VALUE)
-  {
-    Warning(0, "Attempt to free already free'd block (File: %s Line: %d).", file, line);
-    return;
-  }
-  else if (node->tag != MEMTAG_VALUE)
-  {
-    Warning(0, "Attempt to free invalid block (File: %s Line: %d).", file, line);
-    return;
-  }
+	if (node->tag == ~MEMTAG_VALUE)
+	{
+		Warning(0, "Attempt to free already free'd block (File: %s Line: %d).", file, line);
+		return;
+	}
+	else if (node->tag != MEMTAG_VALUE)
+	{
+		Warning(0, "Attempt to free invalid block (File: %s Line: %d).", file, line);
+		return;
+	}
 
 #endif
 
 #if defined(MEM_GUARD)
-  memptr = (char *)block + NODESIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-  {
-     if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
-     {
-        Warning(0, "Damaged start guard detected in free'd block (File: %s Line: %d).", file, line);
-        break;
-     }
-  }
-  memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
-  for(i = 0; i < MEM_GUARD_SIZE; i++)
-  {
-     if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
-     {
-        Warning(0, "Damaged end guard detected in free'd block (File: %s Line: %d).", file, line);
-        break;
-     }
-  }
+	memptr = (char *)block + NODESIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+	{
+		if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
+		{
+			Warning(0, "Damaged start guard detected in free'd block (File: %s Line: %d).", file, line);
+			break;
+		}
+	}
+	memptr = (char *)block + ((MEMNODE *)block)->size - MEM_GUARD_SIZE;
+	for(i = 0; i < MEM_GUARD_SIZE; i++)
+	{
+		if(memptr[i] != mem_guard_string[i % mem_guard_string_len])
+		{
+			Warning(0, "Damaged end guard detected in free'd block (File: %s Line: %d).", file, line);
+			break;
+		}
+	}
 #endif
 
 #if defined(MEM_RECLAIM)
-  remove_node(node);
+	remove_node(node);
 #endif
 
 #if defined(MEM_TAG)
-  /* do this After remove_node, so remove_node can check validity of nodes */
-  node->tag = ~node->tag;
+
+	/* do this After remove_node, so remove_node can check validity of nodes */
+	node->tag = ~node->tag;
 #endif
 
 #if defined(MEM_STATS)
-  mem_stats_free(((MEMNODE*)block)->size);
+	mem_stats_free(((MEMNODE*)block)->size);
 #endif
 
-  FREE(block);
+#if defined(MEM_PREFILL)
+	size = ((MEMNODE *)block)->size;
+	memptr = (char *)block + NODESIZE + MEM_GUARD_SIZE;
+	for(i = 0; i < size - NODESIZE - (MEM_GUARD_SIZE * 2); i++)
+		memptr[i] = mem_clear_string[i % mem_clear_string_len];
+#endif
+
+	FREE(block);
 }
 
 
@@ -664,7 +666,7 @@ void pov_free(void *ptr, const char *file, int line)
 void mem_mark()
 {
 #if defined(MEM_RECLAIM)
-  poolno++;
+	poolno++;
 #endif
 }
 
@@ -674,60 +676,60 @@ void mem_mark()
 void mem_release()
 {
 #if defined(MEM_RECLAIM)
-  OStream *f = NULL;
-  MEMNODE *p, *tmp;
-  size_t totsize;
+	OStream *f = NULL;
+	MEMNODE *p, *tmp;
+	size_t totsize;
 
-  p = memlist;
-  totsize = 0;
+	p = memlist;
+	totsize = 0;
 
 #if defined(MEM_TRACE)
-  if (p != NULL && (p->poolno == poolno))
-    f = New_OStream(MEM_LOG_FNAME, POV_File_Data_LOG, true);
+	if (p != NULL && (p->poolno == poolno))
+		f = New_OStream(MEM_LOG_FNAME, POV_File_Data_LOG, true);
 #endif /* MEM_TRACE */
 
-  while (p != NULL && (p->poolno == poolno))
-  {
+	while (p != NULL && (p->poolno == poolno))
+	{
 #if defined(MEM_TRACE)
 
 #if defined(MEM_TAG)
-    if (!mem_check_tag(p))
-      Debug_Info("mem_release(): Memory pointer corrupt!\n");
+		if (!mem_check_tag(p))
+			Debug_Info("mem_release(): Memory pointer corrupt!\n");
 #endif /* MEM_TAG */
 
-    totsize += (p->size - NODESIZE - (MEM_GUARD_SIZE * 2));
-    if (!leak_msg)
-    {
-      Debug_Info("Memory leakage detected, see file '%s' for list\n",MEM_LOG_FNAME);
-      leak_msg = true;
-    }
+		totsize += (p->size - NODESIZE - (MEM_GUARD_SIZE * 2));
+		if (!leak_msg)
+		{
+			Debug_Info("Memory leakage detected, see file '%s' for list\n",MEM_LOG_FNAME);
+			leak_msg = true;
+		}
 
-    if (f != NULL)
-      f->printf("File:%13s  Line:%4d  Size:%lu\n", p->file, p->line, (unsigned long)(p->size - NODESIZE - (MEM_GUARD_SIZE * 2)));
+		if (f != NULL)
+			f->printf("File:%13s  Line:%4d  Size:%lu\n", p->file, p->line, (unsigned long)(p->size - NODESIZE - (MEM_GUARD_SIZE * 2)));
 #endif /* MEM_TRACE */
 
 #if defined(MEM_STATS)
-    mem_stats_free(p->size);
+		mem_stats_free(p->size);
 #endif
 
-    tmp = p;
-    p = p->next;
-    remove_node(tmp);
-    FREE(tmp);
-  }
+		tmp = p;
+		p = p->next;
+		remove_node(tmp);
+		FREE(tmp);
+	}
 
-  if (f != NULL)
-    delete f;
+	if (f != NULL)
+		delete f;
 
-  if (totsize > 0)
-    Debug_Info("%lu bytes reclaimed (pool #%d)\n", totsize, poolno);
+//	if (totsize > 0)
+//		Debug_Info("%lu bytes reclaimed (pool #%d)\n", totsize, poolno);
 
-  if (poolno > 0)
-    poolno--;
+	if (poolno > 0)
+		poolno--;
 
 #if defined(MEM_STATS)
-  /* reinitialize the stats structure for next time through */
-  mem_stats_init();
+	/* reinitialize the stats structure for next time through */
+	mem_stats_init();
 #endif
 
 #endif /* MEM_RECLAIM */
@@ -739,65 +741,65 @@ void mem_release()
 void mem_release_all()
 {
 #if defined(MEM_RECLAIM)
-  OStream *f = NULL;
-  MEMNODE *p, *tmp;
-  size_t totsize;
+	OStream *f = NULL;
+	MEMNODE *p, *tmp;
+	size_t totsize;
 
-  Send_Progress("Reclaiming memory", PROGRESS_RECLAIMING_MEMORY);
+//  Send_Progress("Reclaiming memory", PROGRESS_RECLAIMING_MEMORY);
 
-  p = memlist;
-  totsize = 0;
+	p = memlist;
+	totsize = 0;
 
 #if defined(MEM_TRACE)
-  if (p != NULL)
-    f = New_OStream(MEM_LOG_FNAME, POV_File_Data_LOG, true);
+	if (p != NULL)
+		f = New_OStream(MEM_LOG_FNAME, POV_File_Data_LOG, true);
 #endif
 
-  while (p != NULL)
-  {
+	while (p != NULL)
+	{
 #if defined(MEM_TRACE)
 
-    #if defined(MEM_TAG)
-    if (!mem_check_tag(p))
-      Debug_Info("mem_release_all(): Memory pointer corrupt!\n");
-    #endif /* MEM_TAG */
+		#if defined(MEM_TAG)
+		if (!mem_check_tag(p))
+			Debug_Info("mem_release_all(): Memory pointer corrupt!\n");
+		#endif /* MEM_TAG */
 
-    totsize += (p->size - NODESIZE - (MEM_GUARD_SIZE * 2));
-    if (!leak_msg)
-    {
-      Debug_Info("Memory leakage detected, see file '%s' for list\n",MEM_LOG_FNAME);
-      leak_msg = true;
-    }
+		totsize += (p->size - NODESIZE - (MEM_GUARD_SIZE * 2));
+		if (!leak_msg)
+		{
+			Debug_Info("Memory leakage detected, see file '%s' for list\n",MEM_LOG_FNAME);
+			leak_msg = true;
+		}
 
-    if (f != NULL)
-      f->printf("File:%13s  Line:%4d  Size:%lu\n", p->file, p->line, (unsigned long)(p->size - NODESIZE - (MEM_GUARD_SIZE * 2)));
+		if (f != NULL)
+			f->printf("File:%13s  Line:%4d  Size:%lu\n", p->file, p->line, (unsigned long)(p->size - NODESIZE - (MEM_GUARD_SIZE * 2)));
 #endif
 
 #if defined(MEM_STATS)
-    /* This is after we have printed stats, and this may slow us down a little,      */
-    /* so we may want to simply re-initialize the mem-stats at the end of this loop. */
-    mem_stats_free(p->size);
+		/* This is after we have printed stats, and this may slow us down a little,      */
+		/* so we may want to simply re-initialize the mem-stats at the end of this loop. */
+		mem_stats_free(p->size);
 #endif
 
-    tmp = p;
-    p = p->next;
-    remove_node(tmp);
-    FREE(tmp);
-  }
+		tmp = p;
+		p = p->next;
+		remove_node(tmp);
+		FREE(tmp);
+	}
 
-  if (f != NULL)
-    delete f;
+	if (f != NULL)
+		delete f;
 
-  if (totsize > 0)
-    Debug_Info("\n%lu bytes reclaimed\n", totsize);
+//	if (totsize > 0)
+//		Debug_Info("\n%lu bytes reclaimed\n", totsize);
 
-  poolno = 0;
-  memlist = NULL;
+	poolno = 0;
+	memlist = NULL;
 #endif
 
 #if defined(MEM_STATS)
-  /* reinitialize the stats structure for next time through */
-  mem_stats_init();
+	/* reinitialize the stats structure for next time through */
+	mem_stats_init();
 #endif
 
 }
@@ -810,27 +812,27 @@ static void add_node(MEMNODE *node)
 {
 
 #if defined(MEM_TAG)
-  if (!mem_check_tag(node))
-    Debug_Info("add_node(): Memory pointer corrupt!\n");
+	if (!mem_check_tag(node))
+		Debug_Info("add_node(): Memory pointer corrupt!\n");
 #endif /* MEM_TAG */
 
-  if (memlist == NULL)
-  {
-    memlist = node;
-    node->poolno = poolno;
-    node->prev = NULL;
-    node->next = NULL;
-    num_nodes = 0;
-  }
-  else
-  {
-    memlist->prev = node;
-    node->poolno = poolno;
-    node->prev = NULL;
-    node->next = memlist;
-    memlist = node;
-  }
-  num_nodes++;
+	if (memlist == NULL)
+	{
+		memlist = node;
+		node->poolno = poolno;
+		node->prev = NULL;
+		node->next = NULL;
+		num_nodes = 0;
+	}
+	else
+	{
+		memlist->prev = node;
+		node->poolno = poolno;
+		node->prev = NULL;
+		node->next = memlist;
+		memlist = node;
+	}
+	num_nodes++;
 }
 
 
@@ -840,30 +842,32 @@ static void remove_node(MEMNODE *node)
 {
 
 #if defined(MEM_TAG)
-  if (!mem_check_tag(node))
-    Debug_Info("remove_node(): Memory pointer corrupt!\n");
+	if (!mem_check_tag(node))
+		Debug_Info("remove_node(): Memory pointer corrupt!\n");
 #endif /* MEM_TAG */
 
-  num_nodes--;
-  if (node->prev != NULL)
-    node->prev->next = node->next;
+	num_nodes--;
+	if (node->prev != NULL)
+		node->prev->next = node->next;
 
-  if (node->next != NULL)
-    node->next->prev = node->prev;
+	if (node->next != NULL)
+		node->next->prev = node->prev;
 
-  if (memlist == node)
-  {
+	if (memlist == node)
+	{
 #if defined(MEM_TAG)
-    /* check node->next if it is non-null, to insure it is safe to assign. */
-    /* if it is null, it is safe since it is the last in the list. */
-    if (node->next)
-      if (!mem_check_tag(node->next))
-        Debug_Info("remove_node(): memlist pointer corrupt!\n");
+		/* check node->next if it is non-null, to insure it is safe to assign. */
+		/* if it is null, it is safe since it is the last in the list. */
+		if (node->next)
+			if (!mem_check_tag(node->next))
+				Debug_Info("remove_node(): memlist pointer corrupt!\n");
 #endif /* MEM_TAG */
 
-    memlist = node->next;
-  }
+		memlist = node->next;
+	}
 
+	node->prev = NULL;
+	node->next = NULL;
 }
 
 #endif /* MEM_RECLAIM */
@@ -874,11 +878,11 @@ static void remove_node(MEMNODE *node)
 /****************************************************************************/
 char *pov_strdup(const char *s)
 {
-  char *New;
-  
-  New=(char *)POV_MALLOC(strlen(s)+1,s);
-  strcpy(New,s);
-  return (New);
+	char *New;
+
+	New=(char *)POV_MALLOC(strlen(s)+1,s);
+	strcpy(New,s);
+	return (New);
 }
 
 /****************************************************************************/
@@ -887,50 +891,50 @@ char *pov_strdup(const char *s)
 
 void *pov_memmove (void *dest, void  *src, size_t length)
 {
-  char *csrc =(char *)src;
-  char *cdest=(char *)dest;
-  
-  if (csrc < cdest && csrc + length >= cdest)
-  {
-    size_t size = cdest - csrc;
+	char *csrc =(char *)src;
+	char *cdest=(char *)dest;
 
-    while (length > 0)
-    {
-      POV_MEMCPY(cdest + length - size, csrc + length - size, size);
+	if (csrc < cdest && csrc + length >= cdest)
+	{
+		size_t size = cdest - csrc;
 
-      length -= size;
+		while (length > 0)
+		{
+			POV_MEMCPY(cdest + length - size, csrc + length - size, size);
 
-      if (length < size)
-        size = length;
-    }
-  }
-  /* I'm not sure if this is needed, but my docs on memcpy say the regions
-   * can't overlap, so theoretically we need to special case this.  If you
-   * don't think it's necessary, you can just comment this part out.
-   */
-  else if (cdest < csrc && cdest + length >= csrc)
-  {
-    char *new_dest = cdest;
-    size_t size = csrc - cdest;
+			length -= size;
 
-    while (length > 0)
-    {
-      POV_MEMCPY(new_dest, csrc, length);
+			if (length < size)
+				size = length;
+		}
+	}
+	/* I'm not sure if this is needed, but my docs on memcpy say the regions
+	 * can't overlap, so theoretically we need to special case this.  If you
+	 * don't think it's necessary, you can just comment this part out.
+	 */
+	else if (cdest < csrc && cdest + length >= csrc)
+	{
+		char *new_dest = cdest;
+		size_t size = csrc - cdest;
 
-      new_dest += size;
-      csrc += size;
-      length -= size;
+		while (length > 0)
+		{
+			POV_MEMCPY(new_dest, csrc, length);
 
-      if (length < size)
-        size = length;
-    }
-  }
-  else
-  {
-    POV_MEMCPY(cdest, csrc, length);
-  }
+			new_dest += size;
+			csrc += size;
+			length -= size;
 
-  return cdest;
+			if (length < size)
+				size = length;
+		}
+	}
+	else
+	{
+		POV_MEMCPY(cdest, csrc, length);
+	}
+
+	return cdest;
 }
 
 
@@ -943,17 +947,17 @@ void *pov_memmove (void *dest, void  *src, size_t length)
 /****************************************************************************/
 static void mem_stats_init()
 {
-  mem_stats.smallest_alloc    = 65535;  /* Must be an unsigned number */
-  mem_stats.largest_alloc     = 0;
-  mem_stats.current_mem_usage = 0;
-  mem_stats.largest_mem_usage = 0;
+	mem_stats.smallest_alloc    = 65535;  /* Must be an unsigned number */
+	mem_stats.largest_alloc     = 0;
+	mem_stats.current_mem_usage = 0;
+	mem_stats.largest_mem_usage = 0;
 #if (MEM_STATS>=2)
-  mem_stats.total_allocs      = 0;
-  mem_stats.total_frees       = 0;
-  mem_stats.largest_file      = "none";
-  mem_stats.largest_line      = -1;
-  mem_stats.smallest_file     = "none";
-  mem_stats.smallest_line     = -1;
+	mem_stats.total_allocs      = 0;
+	mem_stats.total_frees       = 0;
+	mem_stats.largest_file      = "none";
+	mem_stats.largest_line      = -1;
+	mem_stats.smallest_file     = "none";
+	mem_stats.smallest_line     = -1;
 #endif
 }
 
@@ -961,35 +965,35 @@ static void mem_stats_init()
 /* update appropriate fields when an allocation takes place                 */
 static void mem_stats_alloc(size_t nbytes, const char *file, int line)
 {
-  /* update the fields */
-  if (((int) mem_stats.smallest_alloc<0) || (nbytes<mem_stats.smallest_alloc))
-  {
-    mem_stats.smallest_alloc = nbytes;
+	/* update the fields */
+	if (((int) mem_stats.smallest_alloc<0) || (nbytes<mem_stats.smallest_alloc))
+	{
+		mem_stats.smallest_alloc = nbytes;
 #if (MEM_STATS>=2)
-    mem_stats.smallest_file = file;
-    mem_stats.smallest_line = line;
+		mem_stats.smallest_file = file;
+		mem_stats.smallest_line = line;
 #endif
-  }
+	}
 
-  if (nbytes>mem_stats.largest_alloc)
-  {
-    mem_stats.largest_alloc = nbytes;
+	if (nbytes>mem_stats.largest_alloc)
+	{
+		mem_stats.largest_alloc = nbytes;
 #if (MEM_STATS>=2)
-    mem_stats.largest_file = file;
-    mem_stats.largest_line = line;
+		mem_stats.largest_file = file;
+		mem_stats.largest_line = line;
 #endif
-  }
+	}
 
 #if (MEM_STATS>=2)
-  mem_stats.total_allocs++;
+	mem_stats.total_allocs++;
 #endif
 
-  mem_stats.current_mem_usage += nbytes;
+	mem_stats.current_mem_usage += nbytes;
 
-  if (mem_stats.current_mem_usage>mem_stats.largest_mem_usage)
-  {
-    mem_stats.largest_mem_usage = mem_stats.current_mem_usage;
-  }
+	if (mem_stats.current_mem_usage>mem_stats.largest_mem_usage)
+	{
+		mem_stats.largest_mem_usage = mem_stats.current_mem_usage;
+	}
 
 }
 
@@ -997,10 +1001,10 @@ static void mem_stats_alloc(size_t nbytes, const char *file, int line)
 /* update appropriate fields when a free takes place                        */
 static void mem_stats_free(size_t nbytes)
 {
-  /* update the fields */
-  mem_stats.current_mem_usage -= nbytes;
+	/* update the fields */
+	mem_stats.current_mem_usage -= nbytes;
 #if (MEM_STATS>=2)
-  mem_stats.total_frees++;
+	mem_stats.total_frees++;
 #endif
 }
 
@@ -1010,22 +1014,22 @@ static void mem_stats_free(size_t nbytes)
 /****************************************************************************/
 size_t mem_stats_smallest_alloc()
 {
-  return mem_stats.smallest_alloc;
+	return mem_stats.smallest_alloc;
 }
 /****************************************************************************/
 size_t mem_stats_largest_alloc()
 {
-  return mem_stats.largest_alloc;
+	return mem_stats.largest_alloc;
 }
 /****************************************************************************/
 size_t mem_stats_current_mem_usage()
 {
-  return mem_stats.current_mem_usage;
+	return mem_stats.current_mem_usage;
 }
 /****************************************************************************/
 size_t mem_stats_largest_mem_usage()
 {
-  return mem_stats.largest_mem_usage;
+	return mem_stats.largest_mem_usage;
 }
 
 /****************************************************************************/
@@ -1036,36 +1040,36 @@ size_t mem_stats_largest_mem_usage()
 /****************************************************************************/
 const char *mem_stats_smallest_file()
 {
-  return mem_stats.smallest_file;
+	return mem_stats.smallest_file;
 }
 /****************************************************************************/
 int mem_stats_smallest_line()
 {
-  return mem_stats.smallest_line;
+	return mem_stats.smallest_line;
 }
 /****************************************************************************/
 const char *mem_stats_largest_file()
 {
-  return mem_stats.largest_file;
+	return mem_stats.largest_file;
 }
 /****************************************************************************/
 int mem_stats_largest_line()
 {
-  return mem_stats.largest_line;
+	return mem_stats.largest_line;
 }
 /****************************************************************************/
 long int mem_stats_total_allocs()
 {
-  return mem_stats.total_allocs;
+	return mem_stats.total_allocs;
 }
 /****************************************************************************/
 long int mem_stats_total_frees()
 {
-  return mem_stats.total_frees;
+	return mem_stats.total_frees;
 }
 
 #endif
 
 #endif /* MEM_STATS */
 
-END_POV_NAMESPACE
+}
